@@ -28,8 +28,8 @@
 #include "src/phongo_client.h"
 #include "src/phongo_error.h"
 #include "src/phongo_ini.h"
-#include "src/BSON/functions.h"
-#include "src/MongoDB/Monitoring/functions.h"
+#include "src/phongo_log.h"
+#include "src/functions_arginfo.h"
 
 ZEND_DECLARE_MODULE_GLOBALS(mongodb)
 #if defined(ZTS) && defined(COMPILE_DL_MONGODB)
@@ -44,7 +44,6 @@ ZEND_TSRMLS_CACHE_DEFINE();
 static int32_t phongo_num_threads = 0;
 
 /* Declare zend_class_entry dependencies, which are initialized in MINIT */
-zend_class_entry* php_phongo_date_immutable_ce;
 zend_class_entry* php_phongo_json_serializable_ce;
 
 /* {{{ phongo_std_object_handlers */
@@ -91,6 +90,16 @@ PHP_RINIT_FUNCTION(mongodb) /* {{{ */
 		zend_hash_init(MONGODB_G(request_clients), 0, NULL, php_phongo_pclient_destroy_ptr, 0);
 	}
 
+	/* Initialize HashTable for loggers, which is initialized to NULL in GINIT
+	 * and destroyed and reset to NULL in RSHUTDOWN. Since this HashTable will
+	 * store logger object zvals, we specify ZVAL_PTR_DTOR as its element
+	 * destructor so that any still-registered loggers can be freed in
+	 * RSHUTDOWN. */
+	if (MONGODB_G(loggers) == NULL) {
+		ALLOC_HASHTABLE(MONGODB_G(loggers));
+		zend_hash_init(MONGODB_G(loggers), 0, NULL, ZVAL_PTR_DTOR, 0);
+	}
+
 	/* Initialize HashTable for APM subscribers, which is initialized to NULL in
 	 * GINIT and destroyed and reset to NULL in RSHUTDOWN. Since this HashTable
 	 * will store subscriber object zvals, we specify ZVAL_PTR_DTOR as its
@@ -162,6 +171,16 @@ PHP_MINIT_FUNCTION(mongodb) /* {{{ */
 
 	(void) type; /* We don't care if we are loaded via dl() or extension= */
 
+	/* Start by disabling libmongoc's default log handler, which could write to
+	 * stdout/stderr. The PHP driver's log handler may be assigned below when
+	 * parsing INI options or at a later point during a request when registering
+	 * a logger. */
+	mongoc_log_set_handler(NULL, NULL);
+
+	/* Disable trace logging. This will be enabled in phongo_log_sync_handlers()
+	 * if the "mongodb.debug" INI option is set. */
+	mongoc_log_trace_disable();
+
 	phongo_register_ini_entries(INIT_FUNC_ARGS_PASSTHRU);
 
 	/* Assign our custom vtable to libbson, so all memory allocation in libbson
@@ -181,17 +200,12 @@ PHP_MINIT_FUNCTION(mongodb) /* {{{ */
 	phongo_std_object_handlers.get_gc = php_phongo_std_get_gc;
 
 	/* Initialize zend_class_entry dependencies.
-	 *
-	 * Although DateTimeImmutable was introduced in PHP 5.5.0,
-	 * php_date_get_immutable_ce() is not available in PHP versions before
-	 * 5.5.24 and 5.6.8.
 	 *
 	 * Although JsonSerializable was introduced in PHP 5.4.0,
 	 * php_json_serializable_ce is not exported in PHP versions before 5.4.26
 	 * and 5.5.10. For later PHP versions, looking up the class manually also
 	 * helps with distros that disable LTDL_LAZY for dlopen() (e.g. Fedora).
 	 */
-	php_phongo_date_immutable_ce    = php_phongo_fetch_internal_class(ZEND_STRL("datetimeimmutable"));
 	php_phongo_json_serializable_ce = php_phongo_fetch_internal_class(ZEND_STRL("jsonserializable"));
 
 	if (php_phongo_json_serializable_ce == NULL) {
@@ -214,6 +228,10 @@ PHP_MINIT_FUNCTION(mongodb) /* {{{ */
 	php_phongo_timestamp_interface_init_ce(INIT_FUNC_ARGS_PASSTHRU);
 	php_phongo_utcdatetime_interface_init_ce(INIT_FUNC_ARGS_PASSTHRU);
 
+	php_phongo_iterator_init_ce(INIT_FUNC_ARGS_PASSTHRU);
+	php_phongo_packedarray_init_ce(INIT_FUNC_ARGS_PASSTHRU);
+	php_phongo_document_init_ce(INIT_FUNC_ARGS_PASSTHRU);
+
 	php_phongo_binary_init_ce(INIT_FUNC_ARGS_PASSTHRU);
 	php_phongo_dbpointer_init_ce(INIT_FUNC_ARGS_PASSTHRU);
 	php_phongo_decimal128_init_ce(INIT_FUNC_ARGS_PASSTHRU);
@@ -274,6 +292,7 @@ PHP_MINIT_FUNCTION(mongodb) /* {{{ */
 	php_phongo_commandfailedevent_init_ce(INIT_FUNC_ARGS_PASSTHRU);
 	php_phongo_commandstartedevent_init_ce(INIT_FUNC_ARGS_PASSTHRU);
 	php_phongo_commandsucceededevent_init_ce(INIT_FUNC_ARGS_PASSTHRU);
+	php_phongo_logsubscriber_init_ce(INIT_FUNC_ARGS_PASSTHRU);
 	php_phongo_sdamsubscriber_init_ce(INIT_FUNC_ARGS_PASSTHRU);
 	php_phongo_serverchangedevent_init_ce(INIT_FUNC_ARGS_PASSTHRU);
 	php_phongo_serverclosedevent_init_ce(INIT_FUNC_ARGS_PASSTHRU);
@@ -300,6 +319,16 @@ PHP_MSHUTDOWN_FUNCTION(mongodb) /* {{{ */
 
 PHP_RSHUTDOWN_FUNCTION(mongodb) /* {{{ */
 {
+	/* Destroy HashTable for loggers, which was initialized in RINIT. */
+	if (MONGODB_G(loggers)) {
+		zend_hash_destroy(MONGODB_G(loggers));
+		FREE_HASHTABLE(MONGODB_G(loggers));
+		MONGODB_G(loggers) = NULL;
+	}
+
+	/* TODO: consider calling phongo_log_sync_handler here since logging may no
+	 * longer be enabled. */
+
 	/* Destroy HashTable for APM subscribers, which was initialized in RINIT. */
 	if (MONGODB_G(subscribers)) {
 		zend_hash_destroy(MONGODB_G(subscribers));
@@ -337,8 +366,9 @@ PHP_GSHUTDOWN_FUNCTION(mongodb) /* {{{ */
 	 * encryption settings. */
 	zend_hash_graceful_reverse_destroy(&mongodb_globals->persistent_clients);
 
-	phongo_log_disable(mongodb_globals->debug_fd);
-	mongodb_globals->debug_fd = NULL;
+	/* TODO: Check that logging actually gets disabled. The logger HashTable
+	 * should be empty by this point. */
+	phongo_log_set_stream(NULL);
 
 	/* Decrement the thread counter. If it reaches zero, we can infer that this
 	 * is the last thread, MSHUTDOWN has been called, persistent clients from
@@ -415,10 +445,10 @@ PHP_MINFO_FUNCTION(mongodb) /* {{{ */
 	php_info_print_table_row(2, "libmongoc SASL", "disabled");
 #endif
 
-#ifdef MONGOC_ENABLE_ICU
-	php_info_print_table_row(2, "libmongoc ICU", "enabled");
+#ifdef MONGOC_ENABLE_SRV
+	php_info_print_table_row(2, "libmongoc SRV", "enabled");
 #else
-	php_info_print_table_row(2, "libmongoc ICU", "disabled");
+	php_info_print_table_row(2, "libmongoc SRV", "disabled");
 #endif
 
 #ifdef MONGOC_ENABLE_COMPRESSION
@@ -465,6 +495,17 @@ PHP_MINFO_FUNCTION(mongodb) /* {{{ */
 #else /* MONGOCRYPT_ENABLE_CRYPTO */
 	php_info_print_table_row(2, "libmongocrypt crypto", "disabled");
 #endif
+
+	{
+		const char* crypt_shared_version = php_phongo_crypt_shared_version();
+
+		if (crypt_shared_version) {
+			php_info_print_table_row(2, "crypt_shared library version", crypt_shared_version);
+		} else {
+			php_info_print_table_row(2, "crypt_shared library version", "unknown");
+		}
+	}
+
 #else /* MONGOC_ENABLE_CLIENT_SIDE_ENCRYPTION */
 	php_info_print_table_row(2, "libmongocrypt", "disabled");
 #endif
@@ -482,55 +523,13 @@ PHP_FUNCTION(MongoDB_disabled___construct) /* {{{ */
 
 PHP_FUNCTION(MongoDB_disabled___wakeup) /* {{{ */
 {
-	zend_error_handling error_handling;
-
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters_none() == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_NONE();
 
 	phongo_throw_exception(PHONGO_ERROR_RUNTIME, "%s", "MongoDB\\Driver objects cannot be serialized");
 } /* }}} */
 /* }}} */
 
-/* {{{ Extension functions, module dependencies, and module entry */
-ZEND_BEGIN_ARG_INFO_EX(ai_bson_fromPHP, 0, 0, 1)
-	ZEND_ARG_INFO(0, value)
-ZEND_END_ARG_INFO();
-
-ZEND_BEGIN_ARG_INFO_EX(ai_bson_toPHP, 0, 0, 1)
-	ZEND_ARG_INFO(0, bson)
-	ZEND_ARG_ARRAY_INFO(0, typemap, 0)
-ZEND_END_ARG_INFO();
-
-ZEND_BEGIN_ARG_INFO_EX(ai_bson_toJSON, 0, 0, 1)
-	ZEND_ARG_INFO(0, bson)
-ZEND_END_ARG_INFO();
-
-ZEND_BEGIN_ARG_INFO_EX(ai_bson_fromJSON, 0, 0, 1)
-	ZEND_ARG_INFO(0, json)
-ZEND_END_ARG_INFO();
-
-ZEND_BEGIN_ARG_INFO_EX(ai_mongodb_driver_monitoring_subscriber, 0, 0, 1)
-	ZEND_ARG_OBJ_INFO(0, subscriber, MongoDB\\Driver\\Monitoring\\Subscriber, 0)
-ZEND_END_ARG_INFO();
-
-static const zend_function_entry mongodb_functions[] = {
-	/* clang-format off */
-	ZEND_NS_NAMED_FE("MongoDB\\BSON", fromPHP, PHP_FN(MongoDB_BSON_fromPHP), ai_bson_fromPHP)
-	ZEND_NS_NAMED_FE("MongoDB\\BSON", toPHP, PHP_FN(MongoDB_BSON_toPHP), ai_bson_toPHP)
-	ZEND_NS_NAMED_FE("MongoDB\\BSON", toJSON, PHP_FN(MongoDB_BSON_toJSON), ai_bson_toJSON)
-	ZEND_NS_NAMED_FE("MongoDB\\BSON", toCanonicalExtendedJSON, PHP_FN(MongoDB_BSON_toCanonicalExtendedJSON), ai_bson_toJSON)
-	ZEND_NS_NAMED_FE("MongoDB\\BSON", toRelaxedExtendedJSON, PHP_FN(MongoDB_BSON_toRelaxedExtendedJSON), ai_bson_toJSON)
-	ZEND_NS_NAMED_FE("MongoDB\\BSON", fromJSON, PHP_FN(MongoDB_BSON_fromJSON), ai_bson_fromJSON)
-	ZEND_NS_NAMED_FE("MongoDB\\Driver\\Monitoring", addSubscriber, PHP_FN(MongoDB_Driver_Monitoring_addSubscriber), ai_mongodb_driver_monitoring_subscriber)
-	ZEND_NS_NAMED_FE("MongoDB\\Driver\\Monitoring", removeSubscriber, PHP_FN(MongoDB_Driver_Monitoring_removeSubscriber), ai_mongodb_driver_monitoring_subscriber)
-	PHP_FE_END
-	/* clang-format on */
-};
-
+/* {{{ Module dependencies and module entry */
 static const zend_module_dep mongodb_deps[] = {
 	/* clang-format off */
 	ZEND_MOD_REQUIRED("date")
@@ -546,7 +545,7 @@ zend_module_entry mongodb_module_entry = {
 	NULL,
 	mongodb_deps,
 	"mongodb",
-	mongodb_functions,
+	ext_functions,
 	PHP_MINIT(mongodb),
 	PHP_MSHUTDOWN(mongodb),
 	PHP_RINIT(mongodb),
