@@ -157,7 +157,8 @@ static bool php_phongo_bulkwrite_opts_append_array(bson_t* opts, const char* key
 		return false;
 	}
 
-	php_phongo_zval_to_bson(value, PHONGO_BSON_NONE, &b, NULL);
+	// Explicitly allow MongoDB\BSON\PackedArray for array values
+	php_phongo_zval_to_bson(value, PHONGO_BSON_ALLOW_ROOT_ARRAY, &b, NULL);
 
 	if (EG(exception)) {
 		bson_destroy(&b);
@@ -387,7 +388,7 @@ static PHP_METHOD(MongoDB_Driver_BulkWrite, insert)
 	intern = Z_BULKWRITE_OBJ_P(getThis());
 
 	PHONGO_PARSE_PARAMETERS_START(1, 1)
-	PHONGO_PARAM_ARRAY_OR_OBJECT(zdocument)
+	Z_PARAM_ARRAY_OR_OBJECT(zdocument)
 	PHONGO_PARSE_PARAMETERS_END();
 
 	bson_flags |= PHONGO_BSON_RETURN_ID;
@@ -429,8 +430,8 @@ static PHP_METHOD(MongoDB_Driver_BulkWrite, update)
 	intern = Z_BULKWRITE_OBJ_P(getThis());
 
 	PHONGO_PARSE_PARAMETERS_START(2, 3)
-	PHONGO_PARAM_ARRAY_OR_OBJECT(zquery)
-	PHONGO_PARAM_ARRAY_OR_OBJECT(zupdate)
+	Z_PARAM_ARRAY_OR_OBJECT(zquery)
+	Z_PARAM_ARRAY_OR_OBJECT(zupdate)
 	Z_PARAM_OPTIONAL
 	Z_PARAM_ARRAY_OR_NULL(zoptions)
 	PHONGO_PARSE_PARAMETERS_END();
@@ -441,7 +442,8 @@ static PHP_METHOD(MongoDB_Driver_BulkWrite, update)
 		goto cleanup;
 	}
 
-	php_phongo_zval_to_bson(zupdate, PHONGO_BSON_NONE, &bupdate, NULL);
+	// Explicitly allow MongoDB\BSON\PackedArray for update pipelines
+	php_phongo_zval_to_bson(zupdate, PHONGO_BSON_ALLOW_ROOT_ARRAY, &bupdate, NULL);
 
 	if (EG(exception)) {
 		goto cleanup;
@@ -494,7 +496,7 @@ static PHP_METHOD(MongoDB_Driver_BulkWrite, delete)
 	intern = Z_BULKWRITE_OBJ_P(getThis());
 
 	PHONGO_PARSE_PARAMETERS_START(1, 2)
-	PHONGO_PARAM_ARRAY_OR_OBJECT(zquery)
+	Z_PARAM_ARRAY_OR_OBJECT(zquery)
 	Z_PARAM_OPTIONAL
 	Z_PARAM_ARRAY_OR_NULL(zoptions)
 	PHONGO_PARSE_PARAMETERS_END();
