@@ -24,6 +24,7 @@
 #include "php_phongo.h"
 #include "phongo_bson_encode.h"
 #include "phongo_error.h"
+#include "Command_arginfo.h"
 
 zend_class_entry* php_phongo_command_ce;
 
@@ -32,7 +33,7 @@ zend_class_entry* php_phongo_command_ce;
  *
  * The "maxAwaitTimeMS" option is assigned to the cursor after query execution
  * via mongoc_cursor_set_max_await_time_ms(). */
-static bool php_phongo_command_init_max_await_time_ms(php_phongo_command_t* intern, zval* options) /* {{{ */
+static bool php_phongo_command_init_max_await_time_ms(php_phongo_command_t* intern, zval* options)
 {
 	int64_t max_await_time_ms;
 
@@ -55,12 +56,12 @@ static bool php_phongo_command_init_max_await_time_ms(php_phongo_command_t* inte
 	intern->max_await_time_ms = (uint32_t) max_await_time_ms;
 
 	return true;
-} /* }}} */
+}
 
 /* Initializes the php_phongo_command_init from options argument. This
  * function will fall back to a modifier in the absence of a top-level option
  * (where applicable). */
-static bool php_phongo_command_init(php_phongo_command_t* intern, zval* filter, zval* options) /* {{{ */
+static bool php_phongo_command_init(php_phongo_command_t* intern, zval* filter, zval* options)
 {
 	bson_iter_t iter;
 	bson_iter_t sub_iter;
@@ -94,51 +95,32 @@ static bool php_phongo_command_init(php_phongo_command_t* intern, zval* filter,
 	}
 
 	return true;
-} /* }}} */
+}
 
-/* {{{ proto void MongoDB\Driver\Command::__construct(array|object $document[, array $options = array()])
-   Constructs a new Command */
-static PHP_METHOD(Command, __construct)
+/* Constructs a new Command */
+static PHP_METHOD(MongoDB_Driver_Command, __construct)
 {
-	zend_error_handling   error_handling;
 	php_phongo_command_t* intern;
 	zval*                 document;
 	zval*                 options = NULL;
 
 	intern = Z_COMMAND_OBJ_P(getThis());
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "A|a!", &document, &options) == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_START(1, 2)
+	Z_PARAM_ARRAY_OR_OBJECT(document)
+	Z_PARAM_OPTIONAL
+	Z_PARAM_ARRAY_OR_NULL(options)
+	PHONGO_PARSE_PARAMETERS_END();
 
 	php_phongo_command_init(intern, document, options);
-} /* }}} */
+}
 
-/* {{{ MongoDB\Driver\Command function entries */
-ZEND_BEGIN_ARG_INFO_EX(ai_Command___construct, 0, 0, 1)
-	ZEND_ARG_INFO(0, document)
-	ZEND_ARG_ARRAY_INFO(0, options, 1)
-ZEND_END_ARG_INFO()
+PHONGO_DISABLED_WAKEUP(MongoDB_Driver_Command)
 
-ZEND_BEGIN_ARG_INFO_EX(ai_Command_void, 0, 0, 0)
-ZEND_END_ARG_INFO()
-
-static zend_function_entry php_phongo_command_me[] = {
-	/* clang-format off */
-	PHP_ME(Command, __construct, ai_Command___construct, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	ZEND_NAMED_ME(__wakeup, PHP_FN(MongoDB_disabled___wakeup), ai_Command_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_FE_END
-	/* clang-format on */
-};
-/* }}} */
-
-/* {{{ MongoDB\Driver\Command object handlers */
+/* MongoDB\Driver\Command object handlers */
 static zend_object_handlers php_phongo_handler_command;
 
-static void php_phongo_command_free_object(zend_object* object) /* {{{ */
+static void php_phongo_command_free_object(zend_object* object)
 {
 	php_phongo_command_t* intern = Z_OBJ_COMMAND(object);
 
@@ -147,9 +129,9 @@ static void php_phongo_command_free_object(zend_object* object) /* {{{ */
 	if (intern->bson) {
 		bson_clear(&intern->bson);
 	}
-} /* }}} */
+}
 
-static zend_object* php_phongo_command_create_object(zend_class_entry* class_type) /* {{{ */
+static zend_object* php_phongo_command_create_object(zend_class_entry* class_type)
 {
 	php_phongo_command_t* intern = zend_object_alloc(sizeof(php_phongo_command_t), class_type);
 
@@ -159,9 +141,9 @@ static zend_object* php_phongo_command_create_object(zend_class_entry* class_typ
 	intern->std.handlers = &php_phongo_handler_command;
 
 	return &intern->std;
-} /* }}} */
+}
 
-static HashTable* php_phongo_command_get_debug_info(phongo_compat_object_handler_type* object, int* is_temp) /* {{{ */
+static HashTable* php_phongo_command_get_debug_info(phongo_compat_object_handler_type* object, int* is_temp)
 {
 	php_phongo_command_t* intern;
 	zval                  retval = ZVAL_STATIC_INIT;
@@ -174,7 +156,7 @@ static HashTable* php_phongo_command_get_debug_info(phongo_compat_object_handler
 	if (intern->bson) {
 		zval zv;
 
-		if (!php_phongo_bson_to_zval(bson_get_data(intern->bson), intern->bson->len, &zv)) {
+		if (!php_phongo_bson_to_zval(intern->bson, &zv)) {
 			zval_ptr_dtor(&zv);
 			goto done;
 		}
@@ -186,22 +168,16 @@ static HashTable* php_phongo_command_get_debug_info(phongo_compat_object_handler
 
 done:
 	return Z_ARRVAL(retval);
+}
 
-} /* }}} */
-/* }}} */
-
-void php_phongo_command_init_ce(INIT_FUNC_ARGS) /* {{{ */
+void php_phongo_command_init_ce(INIT_FUNC_ARGS)
 {
-	zend_class_entry ce;
-
-	INIT_NS_CLASS_ENTRY(ce, "MongoDB\\Driver", "Command", php_phongo_command_me);
-	php_phongo_command_ce                = zend_register_internal_class(&ce);
+	php_phongo_command_ce                = register_class_MongoDB_Driver_Command();
 	php_phongo_command_ce->create_object = php_phongo_command_create_object;
-	PHONGO_CE_FINAL(php_phongo_command_ce);
 	PHONGO_CE_DISABLE_SERIALIZATION(php_phongo_command_ce);
 
 	memcpy(&php_phongo_handler_command, phongo_get_std_object_handlers(), sizeof(zend_object_handlers));
 	php_phongo_handler_command.get_debug_info = php_phongo_command_get_debug_info;
 	php_phongo_handler_command.free_obj       = php_phongo_command_free_object;
 	php_phongo_handler_command.offset         = XtOffsetOf(php_phongo_command_t, std);
-} /* }}} */
+}
