@@ -36,6 +36,7 @@
 #include "MongoDB/Server.h"
 #include "MongoDB/Session.h"
 #include "MongoDB/WriteConcern.h"
+#include "Manager_arginfo.h"
 
 #define PHONGO_MANAGER_URI_DEFAULT "mongodb://127.0.0.1/"
 
@@ -60,7 +61,7 @@ zend_class_entry* php_phongo_manager_ce;
  *
  * This handles the merging of any legacy SSL context options and also makes
  * driverOptions suitable for serialization by removing the resource zval. */
-static bool php_phongo_manager_merge_context_options(zval* zdriverOptions) /* {{{ */
+static bool php_phongo_manager_merge_context_options(zval* zdriverOptions)
 {
 	php_stream_context* context;
 	zval *              zcontext, *zcontextOptions;
@@ -98,14 +99,14 @@ static bool php_phongo_manager_merge_context_options(zval* zdriverOptions) /* {{
 	php_array_unsetc(zdriverOptions, "context");
 
 	return true;
-} /* }}} */
+}
 
 /* Prepare authMechanismProperties for BSON encoding by converting a boolean
  * value for the "CANONICALIZE_HOST_NAME" option to a string.
  *
  * Note: URI options are case-insensitive, so we must iterate through the
  * HashTable in order to detect options. */
-static void php_phongo_manager_prep_authmechanismproperties(zval* properties) /* {{{ */
+static void php_phongo_manager_prep_authmechanismproperties(zval* properties)
 {
 	HashTable* ht_data;
 
@@ -137,7 +138,7 @@ static void php_phongo_manager_prep_authmechanismproperties(zval* properties) /*
 		}
 		ZEND_HASH_FOREACH_END();
 	}
-} /* }}} */
+}
 
 /* Prepare URI options for BSON encoding.
  *
@@ -151,7 +152,7 @@ static void php_phongo_manager_prep_authmechanismproperties(zval* properties) /*
  *
  * Note: URI options are case-insensitive, so we must iterate through the
  * HashTable in order to detect options. */
-static void php_phongo_manager_prep_uri_options(zval* options) /* {{{ */
+static void php_phongo_manager_prep_uri_options(zval* options)
 {
 	HashTable* ht_data;
 
@@ -188,7 +189,7 @@ static void php_phongo_manager_prep_uri_options(zval* options) /* {{{ */
 		}
 		ZEND_HASH_FOREACH_END();
 	}
-} /* }}} */
+}
 
 /* Selects a server for an execute method. If "for_writes" is true, a primary
  * will be selected. Otherwise, a read preference will be used to select the
@@ -199,7 +200,7 @@ static void php_phongo_manager_prep_uri_options(zval* options) /* {{{ */
  *
  * On success, server_id will be set and the function will return true;
  * otherwise, false is returned and an exception is thrown. */
-static bool php_phongo_manager_select_server(bool for_writes, bool inherit_read_preference, zval* zreadPreference, zval* zsession, mongoc_client_t* client, uint32_t* server_id) /* {{{ */
+static bool php_phongo_manager_select_server(bool for_writes, bool inherit_read_preference, zval* zreadPreference, zval* zsession, mongoc_client_t* client, uint32_t* server_id)
 {
 	mongoc_server_description_t* selected_server;
 	const mongoc_read_prefs_t*   read_preference = NULL;
@@ -239,13 +240,13 @@ static bool php_phongo_manager_select_server(bool for_writes, bool inherit_read_
 	}
 
 	return false;
-} /* }}} */
+}
 
-/* {{{ proto void MongoDB\Driver\Manager::__construct([string $uri = "mongodb://127.0.0.1/"[, array $options = array()[, array $driverOptions = array()]]])
-   Constructs a new Manager */
-static PHP_METHOD(Manager, __construct)
+PHONGO_DISABLED_WAKEUP(MongoDB_Driver_Manager)
+
+/* Constructs a new Manager */
+static PHP_METHOD(MongoDB_Driver_Manager, __construct)
 {
-	zend_error_handling   error_handling;
 	php_phongo_manager_t* intern;
 	char*                 uri_string     = NULL;
 	size_t                uri_string_len = 0;
@@ -257,12 +258,12 @@ static PHP_METHOD(Manager, __construct)
 	/* Separate the options and driverOptions zvals, since we may end up
 	 * modifying them in php_phongo_manager_prep_uri_options() and
 	 * php_phongo_manager_merge_context_options() below, respectively. */
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "|s!a/!a/!", &uri_string, &uri_string_len, &options, &driverOptions) == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_START(0, 3)
+	Z_PARAM_OPTIONAL
+	Z_PARAM_STRING_OR_NULL(uri_string, uri_string_len)
+	Z_PARAM_ARRAY_EX(options, 1, 1)
+	Z_PARAM_ARRAY_EX(driverOptions, 1, 1)
+	PHONGO_PARSE_PARAMETERS_END();
 
 	if (options) {
 		php_phongo_manager_prep_uri_options(options);
@@ -283,11 +284,10 @@ static PHP_METHOD(Manager, __construct)
 	if (!php_phongo_manager_register(intern)) {
 		phongo_throw_exception(PHONGO_ERROR_UNEXPECTED_VALUE, "Failed to add Manager to internal registry");
 	}
-} /* }}} */
+}
 
-/* {{{ proto void MongoDB\Driver\Manager::addSubscriber(MongoDB\Driver\Monitoring\Subscriber $subscriber)
-   Registers an event subscriber for this Manager */
-static PHP_METHOD(Manager, addSubscriber)
+/* Registers an event subscriber for this Manager */
+static PHP_METHOD(MongoDB_Driver_Manager, addSubscriber)
 {
 	php_phongo_manager_t* intern;
 	zval*                 subscriber;
@@ -296,6 +296,10 @@ static PHP_METHOD(Manager, addSubscriber)
 	Z_PARAM_OBJECT_OF_CLASS(subscriber, php_phongo_subscriber_ce)
 	PHONGO_PARSE_PARAMETERS_END();
 
+	if (instanceof_function(Z_OBJCE_P(subscriber), php_phongo_logsubscriber_ce)) {
+		phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT, "LogSubscriber instances cannot be registered with a Manager");
+	}
+
 	intern = Z_MANAGER_OBJ_P(getThis());
 
 	/* Lazily initialize the subscriber HashTable */
@@ -305,11 +309,10 @@ static PHP_METHOD(Manager, addSubscriber)
 	}
 
 	phongo_apm_add_subscriber(intern->subscribers, subscriber);
-} /* }}} */
+}
 
-/* {{{ proto MongoDB\Driver\ClientEncryption MongoDB\Driver\Manager::createClientEncryption(array $options)
-   Return a ClientEncryption instance */
-static PHP_METHOD(Manager, createClientEncryption)
+/* Return a ClientEncryption instance */
+static PHP_METHOD(MongoDB_Driver_Manager, createClientEncryption)
 {
 	zval* options;
 
@@ -321,13 +324,11 @@ static PHP_METHOD(Manager, createClientEncryption)
 
 	/* An exception will be thrown on error. */
 	phongo_clientencryption_init(Z_CLIENTENCRYPTION_OBJ_P(return_value), options, getThis());
-} /* }}} */
+}
 
-/* {{{ proto MongoDB\Driver\Cursor MongoDB\Driver\Manager::executeCommand(string $db, MongoDB\Driver\Command $command[, array $options = null])
-   Execute a Command */
-static PHP_METHOD(Manager, executeCommand)
+/* Execute a Command */
+static PHP_METHOD(MongoDB_Driver_Manager, executeCommand)
 {
-	zend_error_handling   error_handling;
 	php_phongo_manager_t* intern;
 	char*                 db;
 	size_t                db_len;
@@ -338,12 +339,12 @@ static PHP_METHOD(Manager, executeCommand)
 	zval*                 zsession        = NULL;
 	uint32_t              server_id       = 0;
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "sO|z!", &db, &db_len, &command, php_phongo_command_ce, &options) == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_START(2, 3)
+	Z_PARAM_STRING_OR_NULL(db, db_len)
+	Z_PARAM_OBJECT_OF_CLASS(command, php_phongo_command_ce)
+	Z_PARAM_OPTIONAL
+	Z_PARAM_ZVAL_OR_NULL(options)
+	PHONGO_PARSE_PARAMETERS_END();
 
 	intern = Z_MANAGER_OBJ_P(getThis());
 
@@ -375,13 +376,11 @@ cleanup:
 	if (free_options) {
 		php_phongo_prep_legacy_option_free(options);
 	}
-} /* }}} */
+}
 
-/* {{{ proto MongoDB\Driver\Cursor MongoDB\Driver\Manager::executeReadCommand(string $db, MongoDB\Driver\Command $command[, array $options = null])
-   Execute a ReadCommand */
-static PHP_METHOD(Manager, executeReadCommand)
+/* Execute a ReadCommand */
+static PHP_METHOD(MongoDB_Driver_Manager, executeReadCommand)
 {
-	zend_error_handling   error_handling;
 	php_phongo_manager_t* intern;
 	char*                 db;
 	size_t                db_len;
@@ -391,12 +390,12 @@ static PHP_METHOD(Manager, executeReadCommand)
 	uint32_t              server_id       = 0;
 	zval*                 zsession        = NULL;
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "sO|a!", &db, &db_len, &command, php_phongo_command_ce, &options) == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_START(2, 3)
+	Z_PARAM_STRING_OR_NULL(db, db_len)
+	Z_PARAM_OBJECT_OF_CLASS(command, php_phongo_command_ce)
+	Z_PARAM_OPTIONAL
+	Z_PARAM_ARRAY_OR_NULL(options)
+	PHONGO_PARSE_PARAMETERS_END();
 
 	intern = Z_MANAGER_OBJ_P(getThis());
 
@@ -421,13 +420,11 @@ static PHP_METHOD(Manager, executeReadCommand)
 	PHONGO_RESET_CLIENT_IF_PID_DIFFERS(intern, intern);
 
 	phongo_execute_command(getThis(), PHONGO_COMMAND_READ, db, command, options, server_id, return_value);
-} /* }}} */
+}
 
-/* {{{ proto MongoDB\Driver\Cursor MongoDB\Driver\Manager::executeWriteCommand(string $db, MongoDB\Driver\Command $command[, array $options = null])
-   Execute a WriteCommand */
-static PHP_METHOD(Manager, executeWriteCommand)
+/* Execute a WriteCommand */
+static PHP_METHOD(MongoDB_Driver_Manager, executeWriteCommand)
 {
-	zend_error_handling   error_handling;
 	php_phongo_manager_t* intern;
 	char*                 db;
 	size_t                db_len;
@@ -436,12 +433,12 @@ static PHP_METHOD(Manager, executeWriteCommand)
 	uint32_t              server_id = 0;
 	zval*                 zsession  = NULL;
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "sO|a!", &db, &db_len, &command, php_phongo_command_ce, &options) == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_START(2, 3)
+	Z_PARAM_STRING_OR_NULL(db, db_len)
+	Z_PARAM_OBJECT_OF_CLASS(command, php_phongo_command_ce)
+	Z_PARAM_OPTIONAL
+	Z_PARAM_ARRAY_OR_NULL(options)
+	PHONGO_PARSE_PARAMETERS_END();
 
 	intern = Z_MANAGER_OBJ_P(getThis());
 
@@ -461,13 +458,11 @@ static PHP_METHOD(Manager, executeWriteCommand)
 	PHONGO_RESET_CLIENT_IF_PID_DIFFERS(intern, intern);
 
 	phongo_execute_command(getThis(), PHONGO_COMMAND_WRITE, db, command, options, server_id, return_value);
-} /* }}} */
+}
 
-/* {{{ proto MongoDB\Driver\Cursor MongoDB\Driver\Manager::executeReadWriteCommand(string $db, MongoDB\Driver\Command $command[, array $options = null])
-   Execute a ReadWriteCommand */
-static PHP_METHOD(Manager, executeReadWriteCommand)
+/* Execute a ReadWriteCommand */
+static PHP_METHOD(MongoDB_Driver_Manager, executeReadWriteCommand)
 {
-	zend_error_handling   error_handling;
 	php_phongo_manager_t* intern;
 	char*                 db;
 	size_t                db_len;
@@ -476,12 +471,12 @@ static PHP_METHOD(Manager, executeReadWriteCommand)
 	uint32_t              server_id = 0;
 	zval*                 zsession  = NULL;
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "sO|a!", &db, &db_len, &command, php_phongo_command_ce, &options) == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_START(2, 3)
+	Z_PARAM_STRING_OR_NULL(db, db_len)
+	Z_PARAM_OBJECT_OF_CLASS(command, php_phongo_command_ce)
+	Z_PARAM_OPTIONAL
+	Z_PARAM_ARRAY_OR_NULL(options)
+	PHONGO_PARSE_PARAMETERS_END();
 
 	intern = Z_MANAGER_OBJ_P(getThis());
 
@@ -501,13 +496,11 @@ static PHP_METHOD(Manager, executeReadWriteCommand)
 	PHONGO_RESET_CLIENT_IF_PID_DIFFERS(intern, intern);
 
 	phongo_execute_command(getThis(), PHONGO_COMMAND_READ_WRITE, db, command, options, server_id, return_value);
-} /* }}} */
+}
 
-/* {{{ proto MongoDB\Driver\Cursor MongoDB\Driver\Manager::executeQuery(string $namespace, MongoDB\Driver\Query $query[, array $options = null])
-   Execute a Query */
-static PHP_METHOD(Manager, executeQuery)
+/* Execute a Query */
+static PHP_METHOD(MongoDB_Driver_Manager, executeQuery)
 {
-	zend_error_handling   error_handling;
 	php_phongo_manager_t* intern;
 	char* namespace;
 	size_t   namespace_len;
@@ -518,12 +511,12 @@ static PHP_METHOD(Manager, executeQuery)
 	uint32_t server_id       = 0;
 	zval*    zsession        = NULL;
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "sO|z!", &namespace, &namespace_len, &query, php_phongo_query_ce, &options) == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_START(2, 3)
+	Z_PARAM_STRING_OR_NULL(namespace, namespace_len)
+	Z_PARAM_OBJECT_OF_CLASS(query, php_phongo_query_ce)
+	Z_PARAM_OPTIONAL
+	Z_PARAM_ZVAL_OR_NULL(options)
+	PHONGO_PARSE_PARAMETERS_END();
 
 	intern = Z_MANAGER_OBJ_P(getThis());
 
@@ -555,13 +548,11 @@ cleanup:
 	if (free_options) {
 		php_phongo_prep_legacy_option_free(options);
 	}
-} /* }}} */
+}
 
-/* {{{ proto MongoDB\Driver\WriteResult MongoDB\Driver\Manager::executeBulkWrite(string $namespace, MongoDB\Driver\BulkWrite $zbulk[, array $options = null])
-   Executes a BulkWrite (i.e. any number of insert, update, and delete ops) */
-static PHP_METHOD(Manager, executeBulkWrite)
+/* Executes a BulkWrite (i.e. any number of insert, update, and delete ops) */
+static PHP_METHOD(MongoDB_Driver_Manager, executeBulkWrite)
 {
-	zend_error_handling   error_handling;
 	php_phongo_manager_t* intern;
 	char* namespace;
 	size_t                  namespace_len;
@@ -572,12 +563,12 @@ static PHP_METHOD(Manager, executeBulkWrite)
 	uint32_t                server_id    = 0;
 	zval*                   zsession     = NULL;
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "sO|z!", &namespace, &namespace_len, &zbulk, php_phongo_bulkwrite_ce, &options) == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_START(2, 3)
+	Z_PARAM_STRING_OR_NULL(namespace, namespace_len)
+	Z_PARAM_OBJECT_OF_CLASS(zbulk, php_phongo_bulkwrite_ce)
+	Z_PARAM_OPTIONAL
+	Z_PARAM_ZVAL_OR_NULL(options)
+	PHONGO_PARSE_PARAMETERS_END();
 
 	intern = Z_MANAGER_OBJ_P(getThis());
 	bulk   = Z_BULKWRITE_OBJ_P(zbulk);
@@ -604,11 +595,10 @@ cleanup:
 	if (free_options) {
 		php_phongo_prep_legacy_option_free(options);
 	}
-} /* }}} */
+}
 
-/* {{{ proto array|object|null MongoDB\Driver\Manager::getEncryptedFieldsMap()
-   Returns the autoEncryption.encryptedFieldsMap driver option */
-static PHP_METHOD(Manager, getEncryptedFieldsMap)
+/* Returns the autoEncryption.encryptedFieldsMap driver option */
+static PHP_METHOD(MongoDB_Driver_Manager, getEncryptedFieldsMap)
 {
 	php_phongo_manager_t* intern;
 
@@ -619,63 +609,42 @@ static PHP_METHOD(Manager, getEncryptedFieldsMap)
 	if (!Z_ISUNDEF(intern->enc_fields_map)) {
 		RETURN_ZVAL(&intern->enc_fields_map, 1, 0);
 	}
-} /* }}} */
+}
 
-/* {{{ proto MongoDB\Driver\ReadConcern MongoDB\Driver\Manager::getReadConcern()
-   Returns the ReadConcern associated with this Manager */
-static PHP_METHOD(Manager, getReadConcern)
+/* Returns the ReadConcern associated with this Manager */
+static PHP_METHOD(MongoDB_Driver_Manager, getReadConcern)
 {
-	zend_error_handling   error_handling;
 	php_phongo_manager_t* intern;
 
 	intern = Z_MANAGER_OBJ_P(getThis());
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters_none() == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_NONE();
 
 	phongo_readconcern_init(return_value, mongoc_client_get_read_concern(intern->client));
-} /* }}} */
+}
 
-/* {{{ proto MongoDB\Driver\ReadPreference MongoDB\Driver\Manager::getReadPreference()
-   Returns the ReadPreference associated with this Manager */
-static PHP_METHOD(Manager, getReadPreference)
+/* Returns the ReadPreference associated with this Manager */
+static PHP_METHOD(MongoDB_Driver_Manager, getReadPreference)
 {
-	zend_error_handling   error_handling;
 	php_phongo_manager_t* intern;
 
 	intern = Z_MANAGER_OBJ_P(getThis());
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters_none() == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_NONE();
 
 	phongo_readpreference_init(return_value, mongoc_client_get_read_prefs(intern->client));
-} /* }}} */
+}
 
-/* {{{ proto MongoDB\Driver\Server[] MongoDB\Driver\Manager::getServers()
-   Returns the Servers associated with this Manager */
-static PHP_METHOD(Manager, getServers)
+/* Returns the Servers associated with this Manager */
+static PHP_METHOD(MongoDB_Driver_Manager, getServers)
 {
-	zend_error_handling           error_handling;
 	php_phongo_manager_t*         intern;
 	mongoc_server_description_t** sds;
 	size_t                        i, n = 0;
 
 	intern = Z_MANAGER_OBJ_P(getThis());
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters_none() == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_NONE();
 
 	sds = mongoc_client_get_server_descriptions(intern->client, &n);
 	array_init_size(return_value, n);
@@ -688,30 +657,22 @@ static PHP_METHOD(Manager, getServers)
 	}
 
 	mongoc_server_descriptions_destroy_all(sds, n);
-} /* }}} */
+}
 
-/* {{{ proto MongoDB\Driver\WriteConcern MongoDB\Driver\Manager::getWriteConcern()
-   Returns the WriteConcern associated with this Manager */
-static PHP_METHOD(Manager, getWriteConcern)
+/* Returns the WriteConcern associated with this Manager */
+static PHP_METHOD(MongoDB_Driver_Manager, getWriteConcern)
 {
-	zend_error_handling   error_handling;
 	php_phongo_manager_t* intern;
 
 	intern = Z_MANAGER_OBJ_P(getThis());
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters_none() == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_NONE();
 
 	phongo_writeconcern_init(return_value, mongoc_client_get_write_concern(intern->client));
-} /* }}} */
+}
 
-/* {{{ proto void MongoDB\Driver\Manager::removeSubscriber(MongoDB\Driver\Monitoring\Subscriber $subscriber)
-   Unregisters an event subscriber for this Manager */
-static PHP_METHOD(Manager, removeSubscriber)
+/* Unregisters an event subscriber for this Manager */
+static PHP_METHOD(MongoDB_Driver_Manager, removeSubscriber)
 {
 	php_phongo_manager_t* intern;
 	zval*                 subscriber;
@@ -728,11 +689,10 @@ static PHP_METHOD(Manager, removeSubscriber)
 	}
 
 	phongo_apm_remove_subscriber(intern->subscribers, subscriber);
-} /* }}} */
+}
 
-/* {{{ proto MongoDB\Driver\Server MongoDB\Driver\Manager::selectServers([MongoDB\Driver\ReadPreference $readPreference = null])
-   Selects a Server for the given ReadPreference (default: primary). */
-static PHP_METHOD(Manager, selectServer)
+/* Selects a Server for the given ReadPreference (default: primary). */
+static PHP_METHOD(MongoDB_Driver_Manager, selectServer)
 {
 	php_phongo_manager_t* intern;
 	zval*                 zreadPreference = NULL;
@@ -751,13 +711,11 @@ static PHP_METHOD(Manager, selectServer)
 	}
 
 	phongo_server_init(return_value, getThis(), server_id);
-} /* }}} */
+}
 
-/* {{{ proto MongoDB\Driver\Session MongoDB\Driver\Manager::startSession([array $options = null])
-   Returns a new client session */
-static PHP_METHOD(Manager, startSession)
+/* Returns a new client session */
+static PHP_METHOD(MongoDB_Driver_Manager, startSession)
 {
-	zend_error_handling       error_handling;
 	php_phongo_manager_t*     intern;
 	zval*                     options = NULL;
 	mongoc_session_opt_t*     cs_opts = NULL;
@@ -767,12 +725,10 @@ static PHP_METHOD(Manager, startSession)
 
 	intern = Z_MANAGER_OBJ_P(getThis());
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "|a!", &options) == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_START(0, 1)
+	Z_PARAM_OPTIONAL
+	Z_PARAM_ARRAY_OR_NULL(options)
+	PHONGO_PARSE_PARAMETERS_END();
 
 	if (options && php_array_existsc(options, "causalConsistency")) {
 		cs_opts = mongoc_session_opts_new();
@@ -840,91 +796,12 @@ cleanup:
 	if (cs_opts) {
 		mongoc_session_opts_destroy(cs_opts);
 	}
-} /* }}} */
+}
 
-/* {{{ MongoDB\Driver\Manager function entries */
-ZEND_BEGIN_ARG_INFO_EX(ai_Manager___construct, 0, 0, 0)
-	ZEND_ARG_INFO(0, uri)
-	ZEND_ARG_ARRAY_INFO(0, options, 0)
-	ZEND_ARG_ARRAY_INFO(0, driverOptions, 0)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(ai_Manager_addSubscriber, 0, 0, 1)
-	ZEND_ARG_OBJ_INFO(0, subscriber, MongoDB\\Driver\\Monitoring\\Subscriber, 0)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(ai_Manager_createClientEncryption, 0, 0, 1)
-	ZEND_ARG_ARRAY_INFO(0, options, 0)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(ai_Manager_executeCommand, 0, 0, 2)
-	ZEND_ARG_INFO(0, db)
-	ZEND_ARG_OBJ_INFO(0, command, MongoDB\\Driver\\Command, 0)
-	ZEND_ARG_INFO(0, options)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(ai_Manager_executeRWCommand, 0, 0, 2)
-	ZEND_ARG_INFO(0, db)
-	ZEND_ARG_OBJ_INFO(0, command, MongoDB\\Driver\\Command, 0)
-	ZEND_ARG_ARRAY_INFO(0, options, 0)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(ai_Manager_executeQuery, 0, 0, 2)
-	ZEND_ARG_INFO(0, namespace)
-	ZEND_ARG_OBJ_INFO(0, zquery, MongoDB\\Driver\\Query, 0)
-	ZEND_ARG_INFO(0, options)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(ai_Manager_executeBulkWrite, 0, 0, 2)
-	ZEND_ARG_INFO(0, namespace)
-	ZEND_ARG_OBJ_INFO(0, zbulk, MongoDB\\Driver\\BulkWrite, 0)
-	ZEND_ARG_INFO(0, options)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(ai_Manager_removeSubscriber, 0, 0, 1)
-	ZEND_ARG_OBJ_INFO(0, subscriber, MongoDB\\Driver\\Monitoring\\Subscriber, 0)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(ai_Manager_selectServer, 0, 0, 0)
-	ZEND_ARG_OBJ_INFO(0, readPreference, MongoDB\\Driver\\ReadPreference, 1)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(ai_Manager_startSession, 0, 0, 0)
-	ZEND_ARG_ARRAY_INFO(0, options, 1)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(ai_Manager_void, 0, 0, 0)
-ZEND_END_ARG_INFO()
-
-static zend_function_entry php_phongo_manager_me[] = {
-	/* clang-format off */
-	PHP_ME(Manager, __construct, ai_Manager___construct, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(Manager, addSubscriber, ai_Manager_addSubscriber, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(Manager, createClientEncryption, ai_Manager_createClientEncryption, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(Manager, executeCommand, ai_Manager_executeCommand, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(Manager, executeReadCommand, ai_Manager_executeRWCommand, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(Manager, executeWriteCommand, ai_Manager_executeRWCommand, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(Manager, executeReadWriteCommand, ai_Manager_executeCommand, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(Manager, executeQuery, ai_Manager_executeQuery, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(Manager, executeBulkWrite, ai_Manager_executeBulkWrite, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(Manager, getEncryptedFieldsMap, ai_Manager_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(Manager, getReadConcern, ai_Manager_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(Manager, getReadPreference, ai_Manager_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(Manager, getServers, ai_Manager_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(Manager, getWriteConcern, ai_Manager_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(Manager, removeSubscriber, ai_Manager_removeSubscriber, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(Manager, selectServer, ai_Manager_selectServer, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(Manager, startSession, ai_Manager_startSession, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	ZEND_NAMED_ME(__wakeup, PHP_FN(MongoDB_disabled___wakeup), ai_Manager_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_FE_END
-	/* clang-format on */
-};
-/* }}} */
-
-/* {{{ MongoDB\Driver\Manager object handlers */
+/* MongoDB\Driver\Manager object handlers */
 static zend_object_handlers php_phongo_handler_manager;
 
-static void php_phongo_manager_free_object(zend_object* object) /* {{{ */
+static void php_phongo_manager_free_object(zend_object* object)
 {
 	php_phongo_manager_t* intern = Z_OBJ_MANAGER(object);
 
@@ -966,9 +843,9 @@ static void php_phongo_manager_free_object(zend_object* object) /* {{{ */
 		zend_hash_destroy(intern->subscribers);
 		FREE_HASHTABLE(intern->subscribers);
 	}
-} /* }}} */
+}
 
-static zend_object* php_phongo_manager_create_object(zend_class_entry* class_type) /* {{{ */
+static zend_object* php_phongo_manager_create_object(zend_class_entry* class_type)
 {
 	php_phongo_manager_t* intern = zend_object_alloc(sizeof(php_phongo_manager_t), class_type);
 
@@ -980,9 +857,9 @@ static zend_object* php_phongo_manager_create_object(zend_class_entry* class_typ
 	intern->std.handlers = &php_phongo_handler_manager;
 
 	return &intern->std;
-} /* }}} */
+}
 
-static HashTable* php_phongo_manager_get_debug_info(phongo_compat_object_handler_type* object, int* is_temp) /* {{{ */
+static HashTable* php_phongo_manager_get_debug_info(phongo_compat_object_handler_type* object, int* is_temp)
 {
 	php_phongo_manager_t*         intern;
 	mongoc_server_description_t** sds;
@@ -1020,21 +897,16 @@ done:
 	mongoc_server_descriptions_destroy_all(sds, n);
 
 	return Z_ARRVAL(retval);
-} /* }}} */
-/* }}} */
+}
 
-void php_phongo_manager_init_ce(INIT_FUNC_ARGS) /* {{{ */
+void php_phongo_manager_init_ce(INIT_FUNC_ARGS)
 {
-	zend_class_entry ce;
-
-	INIT_NS_CLASS_ENTRY(ce, "MongoDB\\Driver", "Manager", php_phongo_manager_me);
-	php_phongo_manager_ce                = zend_register_internal_class(&ce);
+	php_phongo_manager_ce                = register_class_MongoDB_Driver_Manager();
 	php_phongo_manager_ce->create_object = php_phongo_manager_create_object;
-	PHONGO_CE_FINAL(php_phongo_manager_ce);
 	PHONGO_CE_DISABLE_SERIALIZATION(php_phongo_manager_ce);
 
 	memcpy(&php_phongo_handler_manager, phongo_get_std_object_handlers(), sizeof(zend_object_handlers));
 	php_phongo_handler_manager.get_debug_info = php_phongo_manager_get_debug_info;
 	php_phongo_handler_manager.free_obj       = php_phongo_manager_free_object;
 	php_phongo_handler_manager.offset         = XtOffsetOf(php_phongo_manager_t, std);
-} /* }}} */
+}
