@@ -26,13 +26,15 @@
 #include "phongo_bson_encode.h"
 #include "phongo_error.h"
 
+#include "MongoDB/Query.h"
 #include "MongoDB/ReadConcern.h"
+#include "Query_arginfo.h"
 
 zend_class_entry* php_phongo_query_ce;
 
 /* Appends a string field into the BSON options. Returns true on success;
  * otherwise, false is returned and an exception is thrown. */
-static bool php_phongo_query_opts_append_string(bson_t* opts, const char* opts_key, zval* zarr, const char* zarr_key) /* {{{ */
+static bool php_phongo_query_opts_append_string(bson_t* opts, const char* opts_key, zval* zarr, const char* zarr_key)
 {
 	zval* value = php_array_fetch(zarr, zarr_key);
 
@@ -47,11 +49,11 @@ static bool php_phongo_query_opts_append_string(bson_t* opts, const char* opts_k
 	}
 
 	return true;
-} /* }}} */
+}
 
 /* Appends a document field for the given opts document and key. Returns true on
  * success; otherwise, false is returned and an exception is thrown. */
-static bool php_phongo_query_opts_append_document(bson_t* opts, const char* opts_key, zval* zarr, const char* zarr_key) /* {{{ */
+static bool php_phongo_query_opts_append_document(bson_t* opts, const char* opts_key, zval* zarr, const char* zarr_key)
 {
 	zval*  value = php_array_fetch(zarr, zarr_key);
 	bson_t b     = BSON_INITIALIZER;
@@ -82,15 +84,15 @@ static bool php_phongo_query_opts_append_document(bson_t* opts, const char* opts
 
 	bson_destroy(&b);
 	return true;
-} /* }}} */
+}
 
 /* Appends an arbitrary BSON value for the given opts document and key. Returns
  * true on success; otherwise, false is returned and an exception is thrown. */
-static bool php_phongo_query_opts_append_value(bson_t* opts, const char* opts_key, zval* zarr, const char* zarr_key) /* {{{ */
+static bool php_phongo_query_opts_append_value(bson_t* opts, const char* opts_key, zval* zarr, const char* zarr_key)
 {
 	bson_value_t value = { 0 };
 
-	php_phongo_zval_to_bson_value(php_array_fetch(zarr, zarr_key), PHONGO_BSON_NONE, &value);
+	phongo_zval_to_bson_value(php_array_fetch(zarr, zarr_key), &value);
 
 	if (EG(exception)) {
 		/* Exception should already have been thrown */
@@ -106,7 +108,7 @@ static bool php_phongo_query_opts_append_value(bson_t* opts, const char* opts_ke
 
 	bson_value_destroy(&value);
 	return true;
-} /* }}} */
+}
 
 #define PHONGO_QUERY_OPT_BOOL_EX(opt, zarr, key, deprecated)                                                                      \
 	if ((zarr) && php_array_existsc((zarr), (key))) {                                                                             \
@@ -165,7 +167,7 @@ static bool php_phongo_query_opts_append_value(bson_t* opts, const char* opts_ke
  *
  * The "hint" option (or "$hint" modifier) must be a string or document. Check
  * for both types and merge into BSON options accordingly. */
-static bool php_phongo_query_init_hint(php_phongo_query_t* intern, zval* options, zval* modifiers) /* {{{ */
+static bool php_phongo_query_init_hint(php_phongo_query_t* intern, zval* options, zval* modifiers)
 {
 	/* The "hint" option (or "$hint" modifier) must be a string or document.
 	 * Check for both types and merge into BSON options accordingly. */
@@ -194,7 +196,7 @@ static bool php_phongo_query_init_hint(php_phongo_query_t* intern, zval* options
 	}
 
 	return true;
-} /* }}} */
+}
 
 /* Initialize the "limit" and "singleBatch" options. Returns true on success;
  * otherwise, false is returned and an exception is thrown.
@@ -202,7 +204,7 @@ static bool php_phongo_query_init_hint(php_phongo_query_t* intern, zval* options
  * mongoc_collection_find_with_opts() requires a non-negative limit. For
  * backwards compatibility, a negative limit should be set as a positive value
  * and default singleBatch to true. */
-static bool php_phongo_query_init_limit_and_singlebatch(php_phongo_query_t* intern, zval* options) /* {{{ */
+static bool php_phongo_query_init_limit_and_singlebatch(php_phongo_query_t* intern, zval* options)
 {
 	if (php_array_fetchc_long(options, "limit") < 0) {
 		zend_long limit = php_array_fetchc_long(options, "limit");
@@ -227,14 +229,14 @@ static bool php_phongo_query_init_limit_and_singlebatch(php_phongo_query_t* inte
 	}
 
 	return true;
-} /* }}} */
+}
 
 /* Initialize the "readConcern" option. Returns true on success; otherwise,
  * false is returned and an exception is thrown.
  *
  * The "readConcern" option should be a MongoDB\Driver\ReadConcern instance,
  * which must be converted to a mongoc_read_concern_t. */
-static bool php_phongo_query_init_readconcern(php_phongo_query_t* intern, zval* options) /* {{{ */
+static bool php_phongo_query_init_readconcern(php_phongo_query_t* intern, zval* options)
 {
 	zval* read_concern;
 
@@ -252,14 +254,14 @@ static bool php_phongo_query_init_readconcern(php_phongo_query_t* intern, zval*
 	intern->read_concern = mongoc_read_concern_copy(phongo_read_concern_from_zval(read_concern));
 
 	return true;
-} /* }}} */
+}
 
 /* Initialize the "maxAwaitTimeMS" option. Returns true on success; otherwise,
  * false is returned and an exception is thrown.
  *
  * The "maxAwaitTimeMS" option is assigned to the cursor after query execution
  * via mongoc_cursor_set_max_await_time_ms(). */
-static bool php_phongo_query_init_max_await_time_ms(php_phongo_query_t* intern, zval* options) /* {{{ */
+static bool php_phongo_query_init_max_await_time_ms(php_phongo_query_t* intern, zval* options)
 {
 	int64_t max_await_time_ms;
 
@@ -282,20 +284,36 @@ static bool php_phongo_query_init_max_await_time_ms(php_phongo_query_t* intern,
 	intern->max_await_time_ms = (uint32_t) max_await_time_ms;
 
 	return true;
-} /* }}} */
+}
 
-/* Initializes the php_phongo_query_t from filter and options arguments. This
- * function will fall back to a modifier in the absence of a top-level option
- * (where applicable). */
-static bool php_phongo_query_init(php_phongo_query_t* intern, zval* filter, zval* options) /* {{{ */
+/* Initializes the query from filter and options arguments and returns whether
+ * an error occurred. If query is undefined, it will be initialized.
+ *
+ * This function will fall back to a modifier in the absence of a top-level
+ * option (where applicable). */
+bool phongo_query_init(zval* return_value, zval* filter, zval* options)
 {
-	zval* modifiers = NULL;
+	php_phongo_query_t* intern;
+	zval*               modifiers = NULL;
+
+	if (Z_ISUNDEF_P(return_value)) {
+		object_init_ex(return_value, php_phongo_query_ce);
+	}
+
+	if (Z_TYPE_P(return_value) != IS_OBJECT || !instanceof_function(Z_OBJCE_P(return_value), php_phongo_query_ce)) {
+		phongo_throw_exception(PHONGO_ERROR_UNEXPECTED_VALUE, "Expected initialization object to be %s, %s given", ZSTR_VAL(php_phongo_query_ce->name), PHONGO_ZVAL_CLASS_OR_TYPE_NAME_P(return_value));
+		return false;
+	}
+
+	intern = Z_QUERY_OBJ_P(return_value);
 
 	intern->filter            = bson_new();
 	intern->opts              = bson_new();
 	intern->max_await_time_ms = 0;
 
-	php_phongo_zval_to_bson(filter, PHONGO_BSON_NONE, intern->filter, NULL);
+	if (filter) {
+		php_phongo_zval_to_bson(filter, PHONGO_BSON_NONE, intern->filter, NULL);
+	}
 
 	/* Note: if any exceptions are thrown, we can simply return as PHP will
 	 * invoke php_phongo_query_free_object to destruct the object. */
@@ -375,7 +393,7 @@ static bool php_phongo_query_init(php_phongo_query_t* intern, zval* filter, zval
 	}
 
 	return true;
-} /* }}} */
+}
 
 #undef PHONGO_QUERY_OPT_BOOL_EX
 #undef PHONGO_QUERY_OPT_BOOL
@@ -387,49 +405,27 @@ static bool php_phongo_query_init(php_phongo_query_t* intern, zval* filter, zval
 #undef PHONGO_QUERY_OPT_INT64_DEPRECATED
 #undef PHONGO_QUERY_OPT_STRING
 
-/* {{{ proto void MongoDB\Driver\Query::__construct(array|object $filter[, array $options = array()])
-   Constructs a new Query */
-static PHP_METHOD(Query, __construct)
+PHONGO_DISABLED_WAKEUP(MongoDB_Driver_Query)
+
+/* Constructs a new Query */
+static PHP_METHOD(MongoDB_Driver_Query, __construct)
 {
-	zend_error_handling error_handling;
-	php_phongo_query_t* intern;
-	zval*               filter;
-	zval*               options = NULL;
+	zval* filter;
+	zval* options = NULL;
 
-	intern = Z_QUERY_OBJ_P(getThis());
+	PHONGO_PARSE_PARAMETERS_START(1, 2)
+	Z_PARAM_ARRAY_OR_OBJECT(filter)
+	Z_PARAM_OPTIONAL
+	Z_PARAM_ARRAY_OR_NULL(options)
+	PHONGO_PARSE_PARAMETERS_END();
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "A|a!", &filter, &options) == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	phongo_query_init(getThis(), filter, options);
+}
 
-	php_phongo_query_init(intern, filter, options);
-} /* }}} */
-
-/* {{{ MongoDB\Driver\Query function entries */
-ZEND_BEGIN_ARG_INFO_EX(ai_Query___construct, 0, 0, 1)
-	ZEND_ARG_INFO(0, filter)
-	ZEND_ARG_ARRAY_INFO(0, options, 1)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(ai_Query_void, 0, 0, 0)
-ZEND_END_ARG_INFO()
-
-static zend_function_entry php_phongo_query_me[] = {
-	/* clang-format off */
-	PHP_ME(Query, __construct, ai_Query___construct, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	ZEND_NAMED_ME(__wakeup, PHP_FN(MongoDB_disabled___wakeup), ai_Query_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_FE_END
-	/* clang-format on */
-};
-/* }}} */
-
-/* {{{ MongoDB\Driver\Query object handlers */
+/* MongoDB\Driver\Query object handlers */
 static zend_object_handlers php_phongo_handler_query;
 
-static void php_phongo_query_free_object(zend_object* object) /* {{{ */
+static void php_phongo_query_free_object(zend_object* object)
 {
 	php_phongo_query_t* intern = Z_OBJ_QUERY(object);
 
@@ -446,9 +442,9 @@ static void php_phongo_query_free_object(zend_object* object) /* {{{ */
 	if (intern->read_concern) {
 		mongoc_read_concern_destroy(intern->read_concern);
 	}
-} /* }}} */
+}
 
-static zend_object* php_phongo_query_create_object(zend_class_entry* class_type) /* {{{ */
+static zend_object* php_phongo_query_create_object(zend_class_entry* class_type)
 {
 	php_phongo_query_t* intern = zend_object_alloc(sizeof(php_phongo_query_t), class_type);
 
@@ -458,9 +454,9 @@ static zend_object* php_phongo_query_create_object(zend_class_entry* class_type)
 	intern->std.handlers = &php_phongo_handler_query;
 
 	return &intern->std;
-} /* }}} */
+}
 
-static HashTable* php_phongo_query_get_debug_info(phongo_compat_object_handler_type* object, int* is_temp) /* {{{ */
+static HashTable* php_phongo_query_get_debug_info(phongo_compat_object_handler_type* object, int* is_temp)
 {
 	php_phongo_query_t* intern;
 	zval                retval = ZVAL_STATIC_INIT;
@@ -475,7 +471,7 @@ static HashTable* php_phongo_query_get_debug_info(phongo_compat_object_handler_t
 	if (intern->filter) {
 		zval zv;
 
-		if (!php_phongo_bson_to_zval(bson_get_data(intern->filter), intern->filter->len, &zv)) {
+		if (!php_phongo_bson_to_zval(intern->filter, &zv)) {
 			zval_ptr_dtor(&zv);
 			goto done;
 		}
@@ -488,7 +484,7 @@ static HashTable* php_phongo_query_get_debug_info(phongo_compat_object_handler_t
 	if (intern->opts) {
 		zval zv;
 
-		if (!php_phongo_bson_to_zval(bson_get_data(intern->opts), intern->opts->len, &zv)) {
+		if (!php_phongo_bson_to_zval(intern->opts, &zv)) {
 			zval_ptr_dtor(&zv);
 			goto done;
 		}
@@ -509,22 +505,16 @@ static HashTable* php_phongo_query_get_debug_info(phongo_compat_object_handler_t
 
 done:
 	return Z_ARRVAL(retval);
+}
 
-} /* }}} */
-/* }}} */
-
-void php_phongo_query_init_ce(INIT_FUNC_ARGS) /* {{{ */
+void php_phongo_query_init_ce(INIT_FUNC_ARGS)
 {
-	zend_class_entry ce;
-
-	INIT_NS_CLASS_ENTRY(ce, "MongoDB\\Driver", "Query", php_phongo_query_me);
-	php_phongo_query_ce                = zend_register_internal_class(&ce);
+	php_phongo_query_ce                = register_class_MongoDB_Driver_Query();
 	php_phongo_query_ce->create_object = php_phongo_query_create_object;
-	PHONGO_CE_FINAL(php_phongo_query_ce);
 	PHONGO_CE_DISABLE_SERIALIZATION(php_phongo_query_ce);
 
 	memcpy(&php_phongo_handler_query, phongo_get_std_object_handlers(), sizeof(zend_object_handlers));
 	php_phongo_handler_query.get_debug_info = php_phongo_query_get_debug_info;
 	php_phongo_handler_query.free_obj       = php_phongo_query_free_object;
 	php_phongo_handler_query.offset         = XtOffsetOf(php_phongo_query_t, std);
-} /* }}} */
+}
