@@ -312,6 +312,58 @@ _mongoc_apply_srv_max_hosts (const mongoc_host_list_t *hl,
    return hl_array;
 }
 
+// `_detect_nongenuine_host` logs if the host string suggests use of CosmosDB or
+// DocumentDB. See DRIVERS-2583 for behavior requirements. Returns true if a
+// CosmosDB or DocumentDB host is detected.
+static bool
+_detect_nongenuine_host (const char *host)
+{
+   char *const host_lowercase = bson_strdup (host);
+
+   mongoc_lowercase (host, host_lowercase);
+
+   if (mongoc_ends_with (host_lowercase, ".cosmos.azure.com")) {
+      MONGOC_INFO (
+         "You appear to be connected to a CosmosDB cluster. For more "
+         "information regarding feature compatibility and support please "
+         "visit https://www.mongodb.com/supportability/cosmosdb");
+      bson_free (host_lowercase);
+      return true;
+   }
+
+   if (mongoc_ends_with (host_lowercase, ".docdb.amazonaws.com") ||
+       mongoc_ends_with (host_lowercase, ".docdb-elastic.amazonaws.com")) {
+      MONGOC_INFO (
+         "You appear to be connected to a DocumentDB cluster. For more "
+         "information regarding feature compatibility and support please "
+         "visit https://www.mongodb.com/supportability/documentdb");
+      bson_free (host_lowercase);
+      return true;
+   }
+
+   bson_free (host_lowercase);
+   return false;
+}
+
+static void
+_detect_nongenuine_hosts (const mongoc_uri_t *uri)
+{
+   const char *srv_hostname = mongoc_uri_get_srv_hostname (uri);
+   if (srv_hostname) {
+      _detect_nongenuine_host (srv_hostname);
+      return;
+   }
+
+   const mongoc_host_list_t *iter;
+
+   LL_FOREACH (mongoc_uri_get_hosts (uri), iter)
+   {
+      if (_detect_nongenuine_host (iter->host)) {
+         return;
+      }
+   }
+}
+
 /*
  *-------------------------------------------------------------------------
  *
@@ -341,6 +393,8 @@ mongoc_topology_new (const mongoc_uri_t *uri, bool single_threaded)
 
    BSON_ASSERT (uri);
 
+   _detect_nongenuine_hosts (uri);
+
 #ifndef MONGOC_ENABLE_CRYPTO
    if (mongoc_uri_get_option_as_bool (
           uri, MONGOC_URI_RETRYWRITES, MONGOC_DEFAULT_RETRYWRITES)) {
@@ -1649,7 +1703,7 @@ _mongoc_topology_push_server_session (mongoc_topology_t *topology,
 bool
 _mongoc_topology_end_sessions_cmd (mongoc_topology_t *topology, bson_t *cmd)
 {
-   bson_t ar;
+   bson_array_builder_t *ar;
    /* Only end up to 10'000 sessions */
    const int ENDED_SESSION_PRUNING_LIMIT = 10000;
    int i = 0;
@@ -1657,15 +1711,12 @@ _mongoc_topology_end_sessions_cmd (mongoc_topology_t *topology, bson_t *cmd)
       mongoc_server_session_pool_get_existing (topology->session_pool);
 
    bson_init (cmd);
-   BSON_APPEND_ARRAY_BEGIN (cmd, "endSessions", &ar);
+   BSON_APPEND_ARRAY_BUILDER_BEGIN (cmd, "endSessions", &ar);
 
    for (; i < ENDED_SESSION_PRUNING_LIMIT && ss != NULL;
         ++i,
         ss = mongoc_server_session_pool_get_existing (topology->session_pool)) {
-      char buf[16];
-      const char *key;
-      bson_uint32_to_string (i, &key, buf, sizeof buf);
-      BSON_APPEND_DOCUMENT (&ar, key, &ss->lsid);
+      bson_array_builder_append_document (ar, &ss->lsid);
       mongoc_server_session_pool_drop (topology->session_pool, ss);
    }
 
@@ -1675,7 +1726,7 @@ _mongoc_topology_end_sessions_cmd (mongoc_topology_t *topology, bson_t *cmd)
       mongoc_server_session_pool_return (topology->session_pool, ss);
    }
 
-   bson_append_array_end (cmd, &ar);
+   bson_append_array_builder_end (cmd, ar);
 
    return i > 0;
 }
@@ -1707,7 +1758,7 @@ _find_topology_version (const bson_t *reply, bson_t *topology_version)
       return;
    }
    bson_iter_document (&iter, &len, &bytes);
-   bson_init_static (topology_version, bytes, len);
+   BSON_ASSERT (bson_init_static (topology_version, bytes, len));
 }
 
 
